#	$Id: bt.lib.mk,v 1.13 2002/09/29 14:13:35 jmmv Exp $
#	$NetBSD: bsd.lib.mk,v 1.205 2002/07/20 08:50:10 yamt Exp $
#	@(#)bsd.lib.mk	8.3 (Berkeley) 4/22/94

.include <bt.init.mk>

.SUFFIXES: .c .cc .cpp .o .h .sh .m4 .y .s .S .so .a .po

# --------------------------------------------------------------------
# Basic targets
# --------------------------------------------------------------------

.PHONY:		_libinstall _libdeinstall
real-install:	_libinstall
real-deinstall:	_libdeinstall

BT_LIB_MKSTATIC?=	yes
BT_LIB_MKPIC?=		yes

BT_LIB_MAJOR?=	0
BT_LIB_MINOR?=	0
BT_LIB_MICRO?=	0

# Set ${BT_LIB_VERSION} to something useful.
.if !defined(BT_LIB_VERSION)
.if defined(BT_LIB_MAJOR) && !empty(BT_LIB_MAJOR)
.if defined(BT_LIB_MINOR) && !empty(BT_LIB_MINOR)
.if defined(BT_LIB_MICRO) && !empty(BT_LIB_MICRO)
BT_LIB_VERSION=${BT_LIB_MAJOR}.${BT_LIB_MINOR}.${BT_LIB_MICRO}
.else
BT_LIB_VERSION=${BT_LIB_MAJOR}.${BT_LIB_MINOR}
.endif
.else
BT_LIB_VERSION=${BT_LIB_MAJOR}
.endif
.endif
.endif

# Determine library names.
_NAMESTATIC?=		lib${BT_LIB}.a
_NAMEPIC?=		lib${BT_LIB}-${BT_LIB_VERSION}.so
_NAMEPIC_MAJOR?=	lib${BT_LIB}-${BT_LIB_MAJOR}.so

# PIC default flags. This WILL NOT work in every system.
# bt_config's check "bt_check_lib_howto" will set this variables
# according to your system.
.if ${BT_LIB_MKPIC} != "no"
BT_FLAGS_CCPIC?=	-fPIC -DPIC
BT_FLAGS_CXXPIC?=	-fPIC -DPIC
BT_FLAGS_LDPIC?=	-Bshareable
.endif

# --------------------------------------------------------------------
# Implicit conversions
# --------------------------------------------------------------------

# Archive objects

.c.o:
	${BT_PROG_CC} ${BT_FLAGS_CPP} ${BT_FLAGS_CC} \
	    -c ${.IMPSRC} -o ${.TARGET}

.cc.o:
	${BT_PROG_CXX} ${BT_FLAGS_CPP} ${BT_FLAGS_CXX} \
	    -c ${.IMPSRC} -o ${.TARGET}

.cpp.o:
	${BT_PROG_CXX} ${BT_FLAGS_CPP} ${BT_FLAGS_CXX} \
	    -c ${.IMPSRC} -o ${.TARGET}

# PIC objects (po)

.c.po:
	${BT_PROG_CC} ${BT_FLAGS_CPP} ${BT_FLAGS_CC} \
	    ${BT_FLAGS_CCPIC} -c ${.IMPSRC} -o ${.TARGET}

.cc.po:
	${BT_PROG_CXX} ${BT_FLAGS_CPP} ${BT_FLAGS_CXX} \
	    ${BT_FLAGS_CXXPIC} -c ${.IMPSRC} -o ${.TARGET}

.cpp.po:
	${BT_PROG_CXX} ${BT_FLAGS_CPP} ${BT_FLAGS_CXX} \
	    ${BT_FLAGS_CXXPIC} -c ${.IMPSRC} -o ${.TARGET}


# --------------------------------------------------------------------
# Main targets
# --------------------------------------------------------------------

# Default sources if not defined.
.if defined(BT_LIB)
BT_SRCS?=	${BT_LIB}.c
.endif

# Determine object files.
.if !empty(BT_SRCS:N*.h)
BT_OBJS+=	${BT_SRCS:N*.h:R:S/$/.o/g}
BT_POBJS+=	${BT_SRCS:N*.h:R:S/$/.po/g}
.endif

# Determine which compiler to use as linker.
.if !empty(BT_SRCS:M*.cc) || !empty(BT_SRCS:M*.cpp)
.if !defined(BT_PROG_CXX) || empty(BT_PROG_CXX)
.abort C++ compiler required, use "bt_check_prog_cxx".
.endif
#_CMD_LINK=	${BT_PROG_CXX}
_CMD_LINK=	ld
.else
.if !defined(BT_PROG_CC) || empty(BT_PROG_CC)
.abort C compiler required, use "bt_check_prog_cc".
.endif
#_CMD_LINK=	${BT_PROG_CC}
_CMD_LINK=	ld
.endif # !empty(BT_SRCS:M*.cc) || !empty(BT_SRCS:M*.cpp)

# If we have archive object files, set archive build.
.if defined(BT_OBJS) && !empty(BT_OBJS)
.NOPATH: ${BT_OBJS} ${_NAMESTATIC} ${BT_SRCS:C/\..$/.c/}

.if ${BT_LIB_MKSTATIC} != "no"
${_NAMESTATIC}: ${BT_OBJS}
.if !commands(${_NAMESTATIC})
	${BT_PROG_AR} r ${_NAMESTATIC} ${BT_OBJS}
.endif
.endif
.endif

# If we have pic object files, set build.
.if defined(BT_POBJS) && !empty(BT_POBJS)
.NOPATH: ${BT_POBJS} ${_NAMEPIC} ${BT_SRCS:C/\..$/.c/}

.if ${BT_LIB_MKPIC} != "no"
${_NAMEPIC}: ${BT_POBJS}
.if !commands(${_NAMEPIC})
	${_CMD_LINK} ${BT_FLAGS_LD} ${BT_FLAGS_LDPIC} \
	    -o ${.TARGET} ${BT_POBJS} ${BT_LIBS}
.endif
.endif
.endif

real-all: depend ${_NAMESTATIC} ${_NAMEPIC}

.if !target(${_NAMESTATIC})
${_NAMESTATIC}:
.endif

.if !target(${_NAMEPIC})
${_NAMEPIC}:
.endif

# --------------------------------------------------------------------
# Installation
# --------------------------------------------------------------------

__archiveinstall: .USE
.if !exists(${BT_DIR_LIB})
	${BT_INSTALL_DIR} ${BT_DIR_LIB}
.endif
	${BT_INSTALL_DATA} ${.ALLSRC} ${.TARGET}
.if defined(BT_PROG_RANLIB) && !empty(BT_PROG_RANLIB)
	${BT_PROG_RANLIB} ${.TARGET}
.endif

.if !target(_libinstall)
_libinstall::

.if ${BT_LIB_MKSTATIC} != "no"
_libinstall:: ${BT_DIR_LIB}/${_NAMESTATIC}
.PRECIOUS:    ${BT_DIR_LIB}/${_NAMESTATIC}
${BT_DIR_LIB}/${_NAMESTATIC}! ${_NAMESTATIC} __archiveinstall
.endif

.if ${BT_LIB_MKPIC} != "no"
_libinstall:: ${BT_DIR_LIB}/${_NAMEPIC}
.PRECIOUS:    ${BT_DIR_LIB}/${_NAMEPIC}
${BT_DIR_LIB}/${_NAMEPIC}! ${_NAMEPIC}
.if !exists(${BT_DIR_LIB})
	${BT_INSTALL_DIR} ${BT_DIR_LIB}
.endif
	${BT_INSTALL_DATA} ${.ALLSRC} ${.TARGET}
	rm -f ${BT_DIR_LIB}/${_NAMEPIC_MAJOR}
	${BT_INSTALL_SYMLINK} ${_NAMEPIC} \
	    ${BT_DIR_LIB}/${_NAMEPIC_MAJOR}
.endif

.endif # !target(_libinstall)

# --------------------------------------------------------------------
# Deinstallation
# --------------------------------------------------------------------

.if !target(_libdeinstall)
_libdeinstall::
.if exists(${BT_DIR_LIB}/${_NAMESTATIC})
	rm -f ${BT_DIR_LIB}/${_NAMESTATIC}
.endif
.if exists(${BT_DIR_LIB}/${_NAMEPIC})
	rm -f ${BT_DIR_LIB}/${_NAMEPIC}
.endif
.if exists(${BT_DIR_LIB}/${_NAMEPIC_MAJOR})
	rm -f ${BT_DIR_LIB}/${_NAMEPIC_MAJOR}
.endif
.endif


BT_FILES_CLEAN+= a.out [Ee]rrs mklog core *.core
.if ${BT_LIB_MKSTATIC} != "no"
BT_FILES_CLEAN+= ${_NAMESTATIC}
BT_FILES_CLEAN+= ${BT_OBJS}
.endif
.if ${BT_LIB_MKPIC} != "no"
BT_FILES_CLEAN+= ${_NAMEPIC}
BT_FILES_CLEAN+= ${BT_POBJS}
.endif

.include <bt.man.mk>
.include <bt.dep.mk>
.include <bt.clean.mk>

${TARGETS}:	# ensure existence
